// ProceduralDllSample.cpp : Defines the entry point for the console application.
//

#include <iostream>
#include <math.h>
#include "EquWin32.h"

#ifdef UNICODE
#pragma comment(lib, "EquWin32W.lib")
#else
#pragma comment(lib, "EquWin32A.lib")
#endif


// Error handler
void  ErrorHandler(int errCode, const TCHAR* message, const TCHAR* message2)
{
	std::cout<<"Error: "<<message<<": "<<message2<<std::endl;
	 throw errCode;
}


//User defined function
TCalc  pow3( TCalc x)
{
	return x*x*x;
}


int _tmain(int argc, _TCHAR* argv[])
{
	TCHAR *mathExp = _T("1.2e-3+x+2.3*(1-sin(y/3.14)^2)-z");
	TCHAR *varList = _T("x,y,z");
	TCalc args[] = {1, 2.5, 1.7e-2};
	TCalc res = TCalc(0);
	EQUHANDLE equHnd = NULL;

	//Initialize EquTranslator algorithm before we can use any function.
	//This function set memory for EquTranslator
	equHnd = ::InitEquTranslator(
		ErrorHandler	//Error callback function
		);

	try
	{
		//Evaluate math expression for a given point "args"
		res = ::Evaluate(equHnd, mathExp, varList, args);

		args[0]=2.3; args[1]=-5;
		res = ::Evaluate(equHnd,_T("x/y+(x^2-y^2)"), _T("x,y"), args);

		//Add user's function to EquTranslator
		::AddFunction(equHnd, _T("p3"), pow3);

		//Calculate set of points for one math expression
		//Build it first
		::Build(equHnd, _T("3.14*p3(x)+sin(y)-sqrt(z+x)"), _T("x,y,z"));
			
		res=0;
		for(int i=0; i< 10000; i++)
		{
			args[0]=args[0]+1;
			args[1]=i+1;
			args[2]=args[0]/args[1];
			//Do calculation for a given point
			res+=::CalcFor(equHnd, args);
		}

		//Another functions to calculate set of points for a given expression
		::Compile(equHnd, mathExp, varList, args);

		res=0;
		for(int i=0; i< 10000; i++)
		{
			args[0]=args[0]+1;
			args[1]=i+1;
			args[2]=args[0]/args[1];
			//Do calculation for a given point
			res+=::Run(equHnd);
		}

	}
	catch(int errCode)
	{
		std::cout<<"Error code: "<<errCode<<std::endl;
	}
		
	//Release resources
	::CloseEquTranslator(equHnd);

	return 0;
}



